<?php

namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\Controller\CrudWithParameterBag;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DataSource\Manager;
use Teknisa\MAN\Services;
use Teknisa\MAN\Utilities;
use \Exception as Exception;
use Zeedhi\Framework\Util\Functions;

class AssinaturaController extends CrudWithParameterBag {
    const selectAssinatura = "select_assinatura";
    const selectParametrizacaoFinanceira = "parametrizacao_financeira";
    const unidades_participantes_assinatura = "unidades_participantes_assinatura";
    const filial_assinatura = "filial_assinatura";
    const forma_pagamento = "forma_pagamento";
    const conta_receber = "conta_receber";
    const conta_corrente = "conta_corrente";
    const tipo_operacao = "tipo_operacao";
    
    public function __construct(ParameterBag $parameterBag, Manager $dataSourceManager, Services\AssinaturaService $assinaturaService, Utilities\Util $util, $environment, Services\FileUploaderService $fileUploaderService) {
        $this->parameterBag = $parameterBag;
        $this->dataSourceManager = $dataSourceManager;
        $this->assinaturaService = $assinaturaService;
        $this->fileUploaderService = $fileUploaderService;
        $this->util = $util;
        $this->environment = $environment;
    }
    
    public function findAssinatura(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = self::selectAssinatura;
        $this->parameterBagColumns = array('NRORG');
        
        parent::find($request, $response);
    }
    
    public function findFilial(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = self::unidades_participantes_assinatura;
        $this->parameterBagColumns = array('NRORG', 'NRFILASSOC', 'NRSERVASS', 'CDNCSERVASS');
        
        parent::find($request, $response);
    }

    public function findParametrizacaoFinanceira(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = self::selectParametrizacaoFinanceira;
        $this->parameterBagColumns = array('NRORG');
        
        parent::find($request, $response);
    }

    public function saveAssinatura(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $rows = $request->getDataSet()->getRows()[0];
            
            if(!isset($rows['URLARQTERMOPDF']) || $rows['URLARQTERMOPDF'] == '') {
                $rows['URLARQTERMOPDF'] = null;
            } else {
                if(isset($rows['URLARQTERMOPDF'][0]['name'])) {
                    $rows['NMARQUIVO'] = $rows['URLARQTERMOPDF'][0]['name'];
                }
                
                if(isset($rows['URLARQTERMOPDF'][0]['path'])) {
                    if(Functions::arrayKeyExists('URLARQTERMOPDF', $rows)) {
                        $this->fileUploaderService->setImageForSave($rows, 'URLARQTERMOPDF', 'URLARQTERMOPDF');
                    }
                } else {
                    $this->fileUploaderService->setImageForSave($rows, 'URLARQTERMOPDF', 'URLARQTERMOPDF');
                }
            }
            
            $rows['URLARQTERMO'] = (!empty($rows['URLARQTERMOPDF'])) ? $rows['URLARQTERMOPDF'] : $rows['URLARQTERMOTXT'];
            
            if($rows['__is_new']) {
                // INSERT
                $rows['NRSERVASS']  = $this->util->newCode('NRSERVASS', '10');
                $rows['NRFILASSOC'] = $this->util->newCode('NRFILASSOC', '10');
                
                $codigoExiste = $this->assinaturaService->verificaCodigo($rows);
                
                if(!empty($codigoExiste)) {
                    throw new \Exception("Código de Serviço já existe!");
                }
                
                $this->assinaturaService->saveAssinatura($rows);
                
                $response->addNotification(new Notification("Dados inseridos com sucesso!", Notification::TYPE_SUCCESS));
            } else {
                // UPDATE
                $this->assinaturaService->updateAssinatura($rows);
                
                $response->addNotification(new Notification("Dados alterados com sucesso!", Notification::TYPE_SUCCESS));
            }
        } catch (Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function deleteAssinatura(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $row = $request->getDataSet()->getRows()[0];
            
            $this->assinaturaService->deleteAssinaturaCascade($row);
        } catch(\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }
    
    public function saveFilial(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach($request->getDataSet()->getRows() as $row) {
                $arrayCodigoFilial = $row['CDFILIAL'];
                
                foreach($arrayCodigoFilial as $cdfilial) {
                    $row['CDFILIAL'] = $cdfilial;
                    
                    $filialExiste = $this->assinaturaService->verificaUnidade($row);
                    
                    if(!empty($filialExiste)) {
                        throw new \Exception("Filial " . $cdfilial . " já cadastrada!");
                    }
                    
                    $this->assinaturaService->saveFilial($row);
                }
            }
            
            $response->addNotification(new Notification("Dados inseridos com sucesso!", Notification::TYPE_SUCCESS));
        } catch (Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteFilial(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach($request->getDataSet()->getRows() as $row) {
                $this->assinaturaService->deleteFilial($row);
            }
        } catch(\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }
    
    public function filialAssinatura(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = self::filial_assinatura;
        $this->parameterBagColumns = array('NRORG');
        
        parent::find($request, $response);
    }
    
    public function formaPagamento(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = self::forma_pagamento;
        $this->parameterBagColumns = array('NRORG');
        
        parent::find($request, $response);
    }
    
    public function contaReceber(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = self::conta_receber;
        $this->parameterBagColumns = array('NRORG');
        
        parent::find($request, $response);
    }
    
    public function contaCorrente(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = self::conta_corrente;
        $this->parameterBagColumns = array('NRORG');
        
        parent::find($request, $response);
    }
    
    public function tipoDeOperacao(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = self::tipo_operacao;
        $this->parameterBagColumns = array('NRORG');
        
        parent::find($request, $response);
    }
    
    public function deleteCheckParametrizacaoFinanceira(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach($request->getDataSet()->getRows() as $row) {
                $this->assinaturaService->deleteParametrizacaoFinanceira($row);
            }
        } catch(\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }
    
    public function deleteParametrizacaoFinanceira(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $row = $request->getDataSet()->getRows()[0];
            
            $this->assinaturaService->deleteParametrizacaoFinanceira($row);
        } catch(\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }
    
    public function saveParametrizacaoFinanceira(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $rows = $request->getDataSet()->getRows()[0];
            
            if($rows['__is_new']) {
                // INSERT
                $rows['NRPARAMFIN']  = $this->util->newCode('NRSERVASS', '10');
                
                $this->assinaturaService->saveParametrizacaoFinanceira($rows);
                
                $response->addNotification(new Notification("Dados inseridos com sucesso!", Notification::TYPE_SUCCESS));
            } else {
                // UPDATE
                $this->assinaturaService->updateParametrizacaoFinanceira($rows);
                
                $response->addNotification(new Notification("Dados alterados com sucesso!", Notification::TYPE_SUCCESS));
            }
        } catch (Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
}